'use strict';

var app = angular.module('buses');

app.service('backend', function($q, utils, _, Stop, Line, Arrival) {

	var obj = {};

	var baseUrl = isCordova && cordova.platformId === 'android' ? 'https://openbus.emtmadrid.es:9443/emt-proxy-server/last/'
							: 'http://miguelfermin.com/proxy/proxy.php?mode=native&url=https://openbus.emtmadrid.es:9443/emt-proxy-server/last/';

	obj.getNearest = function(location) {
		return callService('geo/GetStopsFromXY.php', {
			longitude 	: location.longitude,
			latitude 	: location.latitude,
			Radius 		: 300,
			cultureInfo : '',
			statistics 	: ''
		}).then(function(res) {
			var stops = _.filter(utils.arrayize(res.stop));
			return _.map(stops, function(_stop) {
				return parseStop(_stop);
			});
		});
	}

	obj.getStopDetail = function(stopId) {
		return callService('geo/GetStopsFromStop.php', {
			idStop 		: stopId,
			Radius		: '0'
		}).then(function(res) {
			// console.log('res.stops');
			// console.log(res.stops);
			return parseStop(res.stops);
		})
	}

	obj.getLineDetail = function(lineId) {
		return callService('bus/GetListLines.php', {
			SelectDate 	: getDate(),
			Lines		: lineId
		}).then(function(res) {
			return parseLine(res.resultValues);
		});
	}

	obj.getArriveStop = function(stopId) {
		return callService('geo/GetArriveStop.php', {
			idStop 		: stopId,
			Radius		: '0'

		}).then(function(res) {
			var arrivals = _.filter(utils.arrayize(res.arrives));
			return _.map(arrivals, function(_arrival) {
				return parseArrival(_arrival)
			});
		});
	}

	obj.getLineStops = function(lineId, direction) {

		direction = direction.toUpperCase() == 'A' ? 2 : 1;

		return callService('geo/GetStopsLine.php', {
			line 		: lineId,
			direction	: direction
		}).then(function(res) {

			var stops = _.filter(utils.arrayize(res.stop));
			return _.map(stops, function(_stop) {
				return parseStop(_stop);
			});

			return stops;
		});
	}

	obj.getAllLines = function() {
		
		return callService('bus/GetListLines.php', {
			SelectDate 	: getDate(),
			Lines		: ''
		}).then(function(res) {
			console.log('GetListLines');
			console.log(res);

			return _.map(res.resultValues, function(_line) {
				return parseLine(_line);
			});
		});
	}

	function padNumber(num) {
		if (num < 10) {
			return '0' + num;
		}
		return num;
	}

	function getDate() {
		var d = new Date();

		return padNumber(d.getDay()) 
			+ '/' + padNumber(d.getMonth()+1) 
			+ '/' + d.getFullYear();
	}

	function callService(endpoint, params) {
		var deferred = $q.defer();

		var data = signParams(params);

		$.post(baseUrl + endpoint, data, function(res) {
			deferred.resolve(res);
		}, 'json');

		return deferred.promise;
	}

	function signParams(data) {
		data.idClient = 'WEB.PORTALMOVIL.IPHONE';
		data.passKey = '2E4D2838-D4B2-4A4B-B844-B977D8817CF9';
		return data;
	}

	function parseStop(_stop) {
		if (!_stop) { return null; };
		
		var stop = new Stop();
		stop.id 		= _stop.stopId;
		stop.name 		= _stop.name;
		stop.latitude 	= _stop.latitude;
		stop.longitude 	= _stop.longitude;
		stop.address  	= _stop.postalAddress;
		stop.lines  	= _.filter(utils.arrayize(_stop.line));

		stop.lines 		= _.map(stop.lines, function(_line) {
			return parseLine(_line);
		});

		return stop;
	}

	function parseLine(_line) {
		var line = new Line();

		line.id 	 = _.find([_line.name, _line.line]);
		line.label 	 = _.find([_line.label, _line.line, _line.name]);

		line.headerA = _.find([_line.headerA, _line.nameA]);
		line.headerB = _.find([_line.headerB, _line.nameB]);

		line.direction 	= _line.direction;

		return line;
	}

	function parseArrival(_arrival) {
		var arrival = new Arrival();

		arrival.timeLeft 	= _arrival.busTimeLeft;
		arrival.lineId 		= _arrival.lineId;

		return arrival;
	}

	return obj;

});
