'use strict';

var app = angular.module('buses');

app.controller('NearestCtrl', function($scope, $rootScope, globalState, backend, utils, location) {

	$scope.load = function() {
		var lastLocation = location.getLastLocation();
		if (!lastLocation) { return false; };

		$scope.loading = true;

		NProgress.start();
		backend.getNearest(lastLocation).then(function(res) {
			NProgress.done();
			$scope.loading = false;
			$scope.stops = res;

			globalState.stops = res;
		}, function() {
			$scope.loading = false;
			NProgress.done();
		});
	}

	$scope.refreshLocation = function() {
		$scope.loading = true;

		NProgress.start();
		location.getLocation().then(function() {
			NProgress.done();
			$scope.loading = false;
			$scope.load();

		}, function() {
			$scope.loading = false;
			NProgress.done();
			alert('¡Venga ya! Ha ocurrido un error obteniendo tu ubicación.');
		});

		$scope.load();
	}

	if (globalState.stops) {
		$scope.stops = globalState.stops;
	} else {
		$scope.refreshLocation();
	}

	analytics.trackView('Nearest stops');

});

app.controller('FavoritesCtrl', function($scope, favoritesManager) {
	$scope.stops = favoritesManager.list();

	analytics.trackView('Favorite stops');
});

app.controller('SearchCtrl', function($scope, $location, globalState, backend) {
	$scope.find = function() {
		NProgress.start();
		backend.getStopDetail($scope.stopId).then(function(stop) {
			NProgress.done();
			if (!stop) {
				alert('No se encuentra ninguna parada con el código indicado');
				return;
			};

			globalState.selectedStop = stop;

			$location.path('/stop/' + stop.id)
		});
	}

	analytics.trackView('Search stop');
});


app.controller('StopDetailCtrl', function($scope, $routeParams, $interval, $location, uiGmapGoogleMapApi, globalState, backend, utils, favoritesManager) {

	var interval;
	$scope.section = 'buses';
	$scope.loading = false;
	$scope.map = {
		center: {latitude: 40.4169359, longitude: -3.7034397},
		zoom: 17
	}
	$scope.markerCoords = {}

	// Functions
	$scope.isFav = favoritesManager.isFav;
	$scope.getAlias = favoritesManager.getAlias;
	$scope.parseInt = parseInt;

	$scope.showMap = function() {
		$scope.section = 'map';
	}

	$scope.showBuses = function() {
		$scope.section = 'buses';
	}

	$scope.gotoLine = function(line) {
		globalState.selectedLine = line;
		$location.path('/lines/' + line.id + '/' + line.direction);
	}

	$scope.toggleFavorite = function() {
		if (favoritesManager.isFav($scope.stop)) {
			favoritesManager.remove($scope.stop);
		} else {
			$scope.stop.alias = prompt('¿Quieres poner un alias a esta parada?');
			favoritesManager.add($scope.stop);
		}
	}

	function loadDetail() {
		$scope.loading = true;
		NProgress.start();
		
		backend.getStopDetail($routeParams.id).then(function(res) {
			NProgress.done();
			$scope.loading = false;

			$scope.stop = res;

			globalState.selectedStop = $scope.stop;

			gotStop();
		});
	}

	function gotStop() {
		$scope.map.center.latitude 		= $scope.stop.latitude;
		$scope.map.center.longitude 	= $scope.stop.longitude;
		$scope.markerCoords.latitude 	= $scope.stop.latitude;
		$scope.markerCoords.longitude 	= $scope.stop.longitude;

		interval = $interval(function() {
			$scope.loadArrives();
		}, 30000);
		$scope.loadArrives();
	}

	$scope.loadArrives = function() {
		$scope.loading = true;
		NProgress.start();
		
		backend.getArriveStop($scope.stop.id).then(function(res) {
			NProgress.done();
			$scope.loading = false;
			$scope.arrivals = res;
		});
	}

	$scope.$on('$destroy', function() {
		$interval.cancel(interval);
    });

    function initialize() {
    	if (globalState.selectedStop && globalState.selectedStop.id == $routeParams.id) {
    		$scope.stop = globalState.selectedStop;
    	}
    	
    	loadDetail();

    	if (typeof AdMob !== 'undefined' && _.random(0, 4) === 0) {
    		setTimeout(function() {
	    		AdMob.showInterstitial();
	    	}, 3000);
    	};

    	analytics.trackView('Stop detail');
    }

    initialize();

});

app.directive('stopList', function($location, globalState, favoritesManager, utils, location) {
	return {
	    scope: {
	      	stops: '='
	    },
	    templateUrl: 'partials/stops/list.html',
	    link: function($scope) {
	    	$scope.distance 		= utils.distance;
	    	$scope.getLastLocation 	= location.getLastLocation;
	    	$scope.isFav 			= favoritesManager.isFav;
			$scope.getAlias 		= favoritesManager.getAlias;

			$scope.gotoStop = function(stop) {
				globalState.selectedStop = stop;
				$location.path('/stop/' + stop.id);
			}

	    }
	}
});